
#pragma once
#ifndef APP_ERROR_H
#define APP_ERROR_H

#include "stdtypes.h"

#include "nrf_sdm.h" // for NRF_FAULT_ID_APP_RANGE_START

#if defined (__GNUC__)
#define STATIC_ASSERT_SIMPLE(EXPR)      _Static_assert(EXPR, "unspecified message")
#define STATIC_ASSERT_MSG(EXPR, MSG)    _Static_assert(EXPR, MSG)
#else
#define STATIC_ASSERT_SIMPLE(EXPR)      static_assert(EXPR, "unspecified message")
#define STATIC_ASSERT_MSG(EXPR, MSG)    static_assert(EXPR, MSG)
#endif

#define _SELECT_ASSERT_FUNC(x, EXPR, MSG, ASSERT_MACRO, ...) ASSERT_MACRO

#define STATIC_ASSERT(...)                                                                          \
    _SELECT_ASSERT_FUNC(x, ##__VA_ARGS__,                                                           \
                        STATIC_ASSERT_MSG(__VA_ARGS__),                                             \
                        STATIC_ASSERT_SIMPLE(__VA_ARGS__))

#define REN_BLE_APP_ASSERT    NRF_FAULT_ID_APP_RANGE_START + 3
#define REN_BLE_APP_ERROR     NRF_FAULT_ID_APP_RANGE_START + 4

/**@brief Structure containing info about an error of the type @ref APP_ASSERT.
 */
typedef struct
{
   uint16_t        line_num;    /**< The line number where the error occurred. */
   uint8_t const * p_file_name; /**< The file in which the error occurred. */
} assert_info_t;

/**@brief Structure containing info about an error of the type @ref NRF_FAULT_ID_SDK_ERROR.
 */
typedef struct
{
   uint32_t        line_num;    /**< The line number where the error occurred. */
   uint8_t const * p_file_name; /**< The file in which the error occurred. */
   uint32_t        err_code;    /**< The error code representing the error that occurred. */
} error_info_t;

extern void app_error_fault_handler(uint32_t id, uint32_t pc, uint32_t info);

//TODO:: possible CException candidate
#define APP_ERROR_CHECK(ERR_CODE)                                                   \
do                                                                                  \
{                                                                                   \
   const uint32_t LOCAL_ERR_CODE = (ERR_CODE);                                      \
   if (LOCAL_ERR_CODE != NRF_SUCCESS)                                               \
   {                                                                                \
      LOG_DEBUG("Error Occurred: 0x%x\r\n",  LOCAL_ERR_CODE);                       \
                                                                                    \
      static error_info_t error_info = {                                            \
               (uint16_t)__LINE__,                                                  \
               (uint8_t *)__FILE__,                                                 \
      };                                                                            \
      error_info.err_code = LOCAL_ERR_CODE;                                         \
      app_error_fault_handler(REN_BLE_APP_ERROR, 0, (uint32_t)(&error_info));       \
   }                                                                                \
} while (0)


//TODO:: possible CException candidate
#define APP_ASSERT(expr)                                                            \
do                                                                                  \
{                                                                                   \
   if (expr)                                                                        \
   {                                                                                \
   }                                                                                \
   else                                                                             \
   {                                                                                \
      static assert_info_t assert_info = {                                          \
                (uint16_t)__LINE__,                                                 \
               (uint8_t *)__FILE__,                                                 \
      };                                                                            \
      app_error_fault_handler(REN_BLE_APP_ASSERT, 0, (uint32_t)(&assert_info));     \
   }                                                                                \
} while(0)

#endif // APP_ERROR_H
