
#pragma once
#ifndef BLE_STACK_H
#define BLE_STACK_H

#include "stdtypes.h"

#include "ble_abstraction.h"
#include "ad_data.h"

#define APP_BLE_CONN_CFG_TAG              1     /**< A tag identifying the SoftDevice BLE configuration. */
#define NRF_SDH_BLE_TOTAL_LINK_COUNT      1
#define NRF_SDH_BLE_PERIPHERAL_LINK_COUNT 0
#define NRF_SDH_BLE_CENTRAL_LINK_COUNT    1
#define NRF_SDH_BLE_GATT_MAX_MTU_SIZE     247
#define NRF_SDH_BLE_VS_UUID_COUNT         2
#define NRF_SDH_BLE_GATTS_ATTR_TAB_SIZE   1408
#define NRF_SDH_BLE_SERVICE_CHANGED       0
#define NRF_SDH_BLE_GAP_DATA_LENGTH       251

typedef struct ble_stack_config_t ble_stack_config_t;

struct ble_stack_config_t {
   uint8_t write_queue_size;
};

typedef void(*ble_stack_evt_handler_t)(const uint16_t conn_handle, const ble_evt_t * const p_ble_evt);

typedef struct ble_stack_params_t ble_stack_params_t;
__PACKED_STRUCT ble_stack_params_t
{
   uint32_t default_adv_interval;
   uint16_t default_adv_duration;
   uint16_t default_conn_interval;
   uint16_t default_event_length;
   uint8_t default_is_extended_evt;
   uint16_t default_slave_latency;
   uint16_t default_conn_sup_timeout;
};

void fnBleStack_Initialize(const ble_stack_config_t * const config);
bool fnBleStack_Subscribe(const uint32_t eventId, const ble_stack_evt_handler_t ble_evt_handler);
bool fnBleStack_Unsubscribe(const uint32_t eventId);

void fnBleStack_StartAdvertising(void);
void fnBleStack_StopAdvertising(void);

void fnBleStack_PollEvents(void);
bool fnBleStack_IsConnected(void);
void fnBleStack_DropConnection(void);
bool fnBleStack_IsMtuSizeEstablished(void);

uint32_t fnBleStack_SendNotification(const uint16_t value_handle, const uint8_t * const p_data, uint16_t * p_length);
bool fnBleStack_SetStackParameters(const ble_stack_params_t * const p_params);
uint32_t fnBleStack_SetGattsValue(uint16_t service_handle, ble_gatts_value_t * p_value);

uint32_t fnBleStack_PrimaryServicesDiscover(uint16_t handle, ble_uuid_t const *  UUID);
uint32_t fnBleStack_ScanStart();
uint32_t fnBleStack_ScanResume();
uint32_t fnBleStack_Connect(ble_gap_addr_t const *p_peer_addr);
uint32_t fnBleStack_GattcWrite(ble_gattc_write_params_t const * write_params);
void fnMtuRequest();

#endif // BLE_STACK_H

