#pragma once
#ifndef FLASH_H
#define FLASH_H

#include "stdtypes.h"

#define NRF52_PAGE_SIZE          4096

/* MSD Flash layout */
#define FLASH_AD_DATA_T_OFFSET               0
#define FLASH_AD_DATA_T_LENGTH               52
#define FLASH_BLE_PARAMS_T_OFFSET            FLASH_AD_DATA_T_OFFSET + FLASH_AD_DATA_T_LENGTH
#define FLASH_BLE_PARAMS_T_LENGTH            15
#define FLASH_CRC32_OFFSET                   FLASH_BLE_PARAMS_T_OFFSET + FLASH_BLE_PARAMS_T_LENGTH

#define FLASH_RF_TEST_ENABLE_OFFSET          NRF52_PAGE_SIZE - 4  // magic number 0xAABBCCDD at the end of the flash page
#define FLASH_RF_TEST_PROTECT_MAGIC_NUM      0xAABBCCDD

typedef __PACKED_STRUCT flash_write_operation_s flash_write_operation_t; //forward declaration - declared in flash.c 

bool fnFlash_IsInitialized(void); /* is manufacturer specific data initialized */

flash_write_operation_t * fnFlash_BeginWrite(const uint32_t addr, uint8_t const * data, const uint32_t length);
void fnFlash_UpdateWriteChecksum(flash_write_operation_t* const op); /* perform crc32 for the outstanding manufacturer specific data write */
bool fnFlash_EndWrite(flash_write_operation_t* const op);

bool fnFlash_Write(const uint32_t addr, uint8_t const * data, const uint32_t length);

extern const uint8_t gabyMSDFlashPage[];

#define MSD_FLASH_START_ADDR                 (uint32_t)gabyMSDFlashPage

#endif // FLASH_H
