#pragma once

#ifndef STD_TYPES_H
#define STD_TYPES_H

#if defined(__ICCARM__)
#pragma language=extended
#endif

// include IAR predefined macros for intellisense - ignored by device target build
#include "predef_macros.h"

#include <stdio.h>
#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>

#include "nrf52.h"
#include "nrf52_bitfields.h"
#include "nrf51_to_nrf52.h"
#include "nrf52_name_change.h"

#include "compiler_abstraction.h"

#define BIT_MASK(nBitNumber)        (1UL << (nBitNumber))

//Logging - using stdio
#if DEBUG
#define LOG_DEBUG printf
#define LOG_DEBUG_BUFFER(pData, s_length)       \
   do                                           \
   {                                            \
      printf("{ ");                             \
      for (uint16_t i = 0; i < s_length; i++)   \
      {                                         \
         printf("0x%x ", ((uint8_t*)pData)[i]); \
      }                                         \
      printf("}\r\n");                          \
   } while(0)

#else
#define LOG_DEBUG(x, ...)
#define LOG_DEBUG_BUFFER(pData, length)   
#endif

#define UNUSED(x) ((void)x)

/** Leaves the minimum of the two 32-bit arguments */
#define MIN(a, b) ((a) < (b) ? (a) : (b))
/** Leaves the maximum of the two 32-bit arguments */
#define MAX(a, b) ((a) < (b) ? (b) : (a))

#define TO_UINT16(buffer, offset)   (uint16_t)((uint16_t)buffer[offset] | ((uint16_t)buffer[offset+1] << 8))
#define TO_UINT32(buffer, offset)   (uint32_t)((uint32_t)buffer[offset] | ((uint32_t)buffer[offset+1] << 8)| ((uint32_t)buffer[offset+2] << 16)| ((uint32_t)buffer[offset+3] << 24))

enum
{
   UNIT_US = 1,               /**< Microseconds resolution */
   UNIT_0_625_MS = 625,       /**< Number of microseconds in 0.625 milliseconds. */
   UNIT_MS = 1000,            /**< Milliseconds resolution */
   UNIT_1_25_MS = 1250,       /**< Number of microseconds in 1.25 milliseconds. */
   UNIT_5_MS = 5000,          /**< Number of microseconds in 10 milliseconds. */
   UNIT_10_MS = 10000         /**< Number of microseconds in 10 milliseconds. */
};
#define MSEC_TO_UNITS(TIME, RESOLUTION) (((TIME) * 1000) / (RESOLUTION)) /* milliseconds to units specific to the softdevice */

typedef int32_t handle_t;

#endif //STD_TYPES_H