
#include "ad_data.h"

#include "ble_abstraction.h"

#define APP_COMPANY_IDENTIFIER          0x0655     /**< Company identifier as per www.bluetooth.org. */

#define AD_TYPE_LENGTH(field_size) (uint16_t)(field_size + 2u) /* +1 for the type +1 for the length */

/* advertising data encoding helpers */
__STATIC_INLINE uint16_t fnGetAdvertisingTotalLength(const ad_data_t * const me);
__STATIC_INLINE uint16_t fnGetScanResponseTotalLength(const ad_data_t * const me);
__STATIC_INLINE uint16_t fnEncode_ADType(const uint8_t type, const uint8_t * const pData, const uint16_t length, /*out*/ uint8_t * const pBuffer);
__STATIC_INLINE uint16_t fnEncode_ManufacturerSpecificADType(const uint16_t company_identifier, const uint8_t * const pData, const uint16_t length, /*out*/ uint8_t * const pBuffer);


 uint16_t fnAdData_GetEncodedAdvertising(const ad_data_t * const me, uint8_t * const pBuffer, const uint16_t count)
{
    uint16_t offset;
    const uint16_t total_length = fnGetAdvertisingTotalLength(me);
    
    if (count < total_length)
       return 0u;

    offset = fnEncode_ADType(BLE_GAP_AD_TYPE_FLAGS, &me->flags, SIZE_OF_FLAGS_FIELD, &pBuffer[0]);
    offset += fnEncode_ADType(BLE_GAP_AD_TYPE_128BIT_SERVICE_UUID_COMPLETE, me->service_uuid, SIZE_OF_UUID_FIELD, &pBuffer[offset]);
    offset += fnEncode_ADType(BLE_GAP_AD_TYPE_COMPLETE_LOCAL_NAME, me->name, fnAdData_GetNameLength(me), &pBuffer[offset]);

    return total_length;
}

 uint16_t fnAdData_GetEncodedScanResponse(const ad_data_t * const me, uint8_t * const pBuffer, const uint16_t count)
 {
    const uint16_t total_length = fnGetScanResponseTotalLength(me);

    if (count < total_length)
       return 0u;

   (void)fnEncode_ManufacturerSpecificADType(APP_COMPANY_IDENTIFIER, me->service_name, strlen((const char*)me->service_name), &pBuffer[0]);

    return total_length;
 }

uint16_t fnAdData_GetNameLength(const ad_data_t * const me)
{
   return strlen((const char*)me->name);
}

 __STATIC_INLINE uint16_t fnGetAdvertisingTotalLength(const ad_data_t * const me)
 {
    return (AD_TYPE_LENGTH(SIZE_OF_FLAGS_FIELD)
       + AD_TYPE_LENGTH(SIZE_OF_UUID_FIELD)      /* TODO:: multiple uuids ??? */
       + AD_TYPE_LENGTH(fnAdData_GetNameLength(me))
       );
 }

 __STATIC_INLINE uint16_t fnGetScanResponseTotalLength(const ad_data_t * const me)
 {
    return AD_TYPE_LENGTH(strlen((const char*)me->service_name) + SIZE_OF_COMPANY_IDENTIFIER);
 }

 __STATIC_INLINE uint16_t fnEncode_ADType(const uint8_t type, const uint8_t * const pData, const uint16_t length, /*out*/ uint8_t * const pBuffer)
 {
    uint16_t offset = 0;
    //length (1 byte)|type (1 byte)|Data (length -1)

    pBuffer[offset++] = length + 1; // +1 for the type
    pBuffer[offset++] = type;

    for (uint16_t i = 0; i < length; i++)
    {
       pBuffer[offset + i] = pData[i];
    }

    return AD_TYPE_LENGTH(length);
 }

 __STATIC_INLINE uint16_t fnEncode_ManufacturerSpecificADType(const uint16_t company_identifier, const uint8_t * const pData, const uint16_t length, /*out*/ uint8_t * const pBuffer)
 {
    uint16_t offset = 0;
    //length (1 byte)|type (1 byte)|Data (length -1)

    pBuffer[offset++] = length + SIZE_OF_COMPANY_IDENTIFIER + 1; // +1 for the type
    pBuffer[offset++] = BLE_GAP_AD_TYPE_MANUFACTURER_SPECIFIC_DATA;
    
    pBuffer[offset++] = (uint8_t)(company_identifier & 0x00ff);
    pBuffer[offset++] = (uint8_t)((company_identifier & 0xff00) >> 8);

    for (uint16_t i = 0; i < length; i++)
    {
       pBuffer[offset + i] = pData[i];
    }

    return AD_TYPE_LENGTH(length + SIZE_OF_COMPANY_IDENTIFIER);
 }

