#include "clock.h"

#define NRF_SDH_CLOCK_LF_RC_CTIV       0
#define NRF_SDH_CLOCK_LF_RC_TEMP_CTIV  0

#define NRFX_DELAY_CPU_FREQ_MHZ 64

__STATIC_INLINE void fnDelay_us(uint32_t time_us);

nrf_clock_lf_cfg_t const clock_lf_cfg =
{
   .source = NRF_CLOCK_LF_SRC_XTAL, // ISP1507 XTAL
   .rc_ctiv = NRF_SDH_CLOCK_LF_RC_CTIV,
   .rc_temp_ctiv = NRF_SDH_CLOCK_LF_RC_TEMP_CTIV,
   .accuracy = NRF_CLOCK_LF_ACCURACY_50_PPM
};

static NRF_CLOCK_Type * gpClock;
static CoreDebug_Type * gpCoreDebug;
static DWT_Type * gpDwt;

void fnClock_Initialise(NRF_CLOCK_Type * const pClock, CoreDebug_Type * const pCoreDebug, DWT_Type * const pDwt)
{
    gpClock = pClock;
    gpCoreDebug = pCoreDebug;
    gpDwt = pDwt;
}

void fnClock_StartLF(void)
{
   gpClock->LFCLKSRC = NRF_CLOCK_LF_SRC_XTAL;
   gpClock->EVENTS_LFCLKSTARTED = 0;
   gpClock->TASKS_LFCLKSTART = 1;

   while (gpClock->EVENTS_LFCLKSTARTED == 0) {}
}

void fnClock_Delay_us(uint32_t delay_us)
{
   fnDelay_us(delay_us);
}

__STATIC_INLINE void fnDelay_us(uint32_t time_us)
{
   if (time_us == 0)
   {
      return;
   }
   uint32_t time_cycles = time_us * NRFX_DELAY_CPU_FREQ_MHZ;

   // Save the current state of the DEMCR register to be able to restore it before exiting
   // this function. Enable the trace and debug blocks (DWT is one of them).
   uint32_t core_debug = gpCoreDebug->DEMCR;
   gpCoreDebug->DEMCR = core_debug | CoreDebug_DEMCR_TRCENA_Msk;

   // Save the current state of the CTRL register in DWT block. Make sure
   // that cycle counter is enabled.
   uint32_t dwt_ctrl = gpDwt->CTRL;
   gpDwt->CTRL = dwt_ctrl | DWT_CTRL_CYCCNTENA_Msk;

   // Store start value of cycle counter.
   uint32_t cyccnt_initial = gpDwt->CYCCNT;

   // Delay required time.
   while ((gpDwt->CYCCNT - cyccnt_initial) < time_cycles)
   {
   }

   // Restore preserved registers.
   gpDwt->CTRL = dwt_ctrl;
   gpCoreDebug->DEMCR = core_debug;
}