
#include "gpio.h"

__STATIC_INLINE void fnInitializeAsDefaultInputPin(const uint32_t pin, uint32_t pull);
__STATIC_INLINE bool fnIsPinSet(const uint32_t pin);

static NRF_GPIO_Type* m_preg;

void fnGPIO_Initialize(NRF_GPIO_Type * const p_reg)
{
    m_preg = p_reg;
}

void fnGPIO_InitializeAsDefaultActiveLowInputPin(const uint32_t pin)
{
   fnInitializeAsDefaultInputPin(pin, GPIO_PIN_CNF_PULL_Pullup);
}

void fnGPIO_InitializeAsDefaultActiveHighInputPin(const uint32_t pin)
{
   fnInitializeAsDefaultInputPin(pin, GPIO_PIN_CNF_PULL_Pulldown);
}

void fnGPIO_InitializeAsDefaultOutputPin(const uint32_t pin)
{
    m_preg->PIN_CNF[pin] = ((uint32_t)GPIO_PIN_CNF_DIR_Output << GPIO_PIN_CNF_DIR_Pos)
      | ((uint32_t)GPIO_PIN_CNF_INPUT_Disconnect << GPIO_PIN_CNF_INPUT_Pos)
      | ((uint32_t)GPIO_PIN_CNF_PULL_Disabled << GPIO_PIN_CNF_PULL_Pos)
      | ((uint32_t)GPIO_PIN_CNF_DRIVE_S0S1 << GPIO_PIN_CNF_DRIVE_Pos)
      | ((uint32_t)GPIO_PIN_CNF_SENSE_Disabled << GPIO_PIN_CNF_SENSE_Pos);
}

void fnGPIO_SetPin(const uint32_t pin)
{
    m_preg->OUTSET = BIT_MASK(pin);
}

void fnGPIO_ClearPin(const uint32_t pin)
{
    m_preg->OUTCLR = BIT_MASK(pin);
}

void fnGPIO_SetPins(uint32_t mask)
{
    m_preg->OUTSET = mask;
}

bool fnGPIO_IsInputPinActive(const uint32_t pin)
{
    if (m_preg->PIN_CNF[pin] && (GPIO_PIN_CNF_PULL_Pullup << GPIO_PIN_CNF_PULL_Pos))
        return !fnIsPinSet(pin);
    else
        return fnIsPinSet(pin);
}

__STATIC_INLINE bool fnIsPinSet(const uint32_t pin)
{
   return (bool)(m_preg->IN & BIT_MASK(pin));
}

__STATIC_INLINE void fnInitializeAsDefaultInputPin(const uint32_t pin, uint32_t pull)
{
    m_preg->PIN_CNF[pin] = ((uint32_t)GPIO_PIN_CNF_DIR_Input << GPIO_PIN_CNF_DIR_Pos)
      | ((uint32_t)GPIO_PIN_CNF_INPUT_Connect << GPIO_PIN_CNF_INPUT_Pos)
      | ((uint32_t)pull << GPIO_PIN_CNF_PULL_Pos)
      | ((uint32_t)GPIO_PIN_CNF_DRIVE_S0S1 << GPIO_PIN_CNF_DRIVE_Pos)
      | ((uint32_t)GPIO_PIN_CNF_SENSE_Disabled << GPIO_PIN_CNF_SENSE_Pos);
}